/*
    The binary expansion of 0.7 is 0.10110011001100 and
    this repeats for ever.

    A cast from double to int causes a truncation of the
    binary representation (not the decimal representation)
    of the double.

    Multiplication by 2 in binary is just a left shift of the significand.

    Multiplication of x by 10 is multiplication by 1010 in binary,
    which is a triple left-shift of x, a left-shift of x, and an
    addition of those two results.
*/

public class PointSeven
{
    public static void main(String[] args)
    {
        double x;
        int y;
        String output = "";

        System.out.println();  //output a blank line

        // Here is a simple example of what can go wrong.
        // Suppose you wanted to convert 4 dollars and 77 cents into pennies.
        x = 4.77;
        y = (int)(100*x);
        System.out.println("$4.77 converts to " + y + " pennies because");
        System.out.println("(int)(100*4.77) becomes " +  y
               + " (before casting it was " + 100*4.77 + ")\n");

        // Put powers of 10 in front of .7
        System.out.println("Powers of 10 added and subtracted from 0.7");
        double power_of_10;
        // loop 16 times since a double has about 16 decimal digits
        for (int i = 0; i <= 16; i++)
        {
            power_of_10 = Math.pow(10.0, i);
            x = (power_of_10 + .7) - power_of_10;  //this always simplifies to 0.7
            System.out.println("(10^" + i + "+.7)-10^" + i + "   becomes\t" + x);
        }

        System.out.println();

        // Put powers of 2 in front of .7 (this makes more sense)
        System.out.println("Powers of 2 added and subtracted from 0.7 (which is 0.1011001100... in binary)");
        double power_of_2;
        // loop 53 times since Java uses a 53 bit mantissa
        for (int i = 0; i <= 53; i++)
        {
            power_of_2 = Math.pow(2.0, i);
            x = (power_of_2 + .7) - power_of_2;   //this always simplifies to .7
            System.out.println("(2^" + i + "+.7)-2^" + i + "\tbecomes   \t" + x);
        }
    }
}
